// Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
//
// Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
// CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//
// See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
//

//  File   : SALOME_Component.idl
//  Author : Alexey Petrov, OPEN CASCADE
//  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/idl/SALOME_GenericObj.idl,v 1.6.2.1.22.4.12.1 2012-04-12 14:04:58 vsr Exp $
//
#ifndef _SALOME_GENERICOBJ_IDL_
#define _SALOME_GENERICOBJ_IDL_

/*! \file SALOME_GenericObj.idl \brief interface for common behavior of %SALOME transient CORBA objects 
*/

module SALOME {

  typedef sequence<string> StringSeq;

  /*!
    \brief interface for common behavior of %SALOME transient CORBA objects 
  */
  interface GenericObj {
    /*! \brief Increase the reference count (mark as used by another object).*/
    void Register();
    /*! \brief Decrease the reference count (release by another object).*/
    void UnRegister();
    /*! \brief Obsolete, left for compatibility reasons only. Use UnRegister() instead.*/
    void Destroy();
  };

  /*!
    \brief common usage interface for exporting of arbitrary data in %SALOME environment
  */
  interface ExportableObject : GenericObj {
    /*! \brief Get a list of supported formats */
    StringSeq GetExportableFormats();
    /*! \brief Export data in the specified format */
    boolean   ExportDataAs(in string format, out GenericObj exporter);
  };
};

#endif
