// Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
//
// Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
// CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//
// See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
//

//  File   : SALOME_Exception.idl
//  Author : Paul RASCLE, EDF
//  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/idl/SALOME_Exception.idl,v 1.8.2.1.22.2.12.1 2012-04-12 14:04:58 vsr Exp $
//
/*! \file SALOME_Exception.idl \brief This file contains the objects defining the main exception used
in %SALOME application. 

The idl SALOME_Exception provides a generic CORBA exception for SALOME,
 with an attribute that gives an exception type,a message, plus optional source file name and line number. 
This idl is intended to serve for all user CORBA exceptions raised in SALOME code, as IDL specification 
does not support exception inheritance. So, all the user CORBA exceptions from SALOME could be 
handled in a single catch. 

*/ 
#ifndef _SALOME_EXCEPTION_IDL_
#define _SALOME_EXCEPTION_IDL_

/*! \brief
Module %SALOME contains all base interfaces of %SALOME Kernel 

*/ 

module SALOME
{
//!  This enumeration contains the elements indicating the type of the exception which can be raised during %SALOME session.
  enum ExceptionType
    { 
      COMM,             /*!< Communication problem */
      BAD_PARAM,        /*!< Bad User parameters */
      INTERNAL_ERROR    /*!< Application level problem, irrecoverable */ 
    };
//!  This struct contains a set of fields defining the structure of the exception.
  struct ExceptionStruct
  {
    ExceptionType type; /*!<Type of the exception.*/
    string        text; /*!<Message which is shown when the exception is raised.*/
    string        sourceFile; /*!<The source file where the exception is raised.*/
    unsigned long lineNumber; /*!<Number of the line of the source file where the exception is raised.*/
  };
/*! \brief The main exception in %SALOME application.

The idl SALOME_Exception provides a generic CORBA exception for SALOME,
 with an attribute that gives an exception type,a message, plus optional source file name and line number.
 This idl is intended to serve for all user CORBA exceptions raised in SALOME code, as IDL specification
 does not support exception inheritance. So, all the user CORBA exceptions from SALOME could be
 handled in a single catch.
*/
  exception SALOME_Exception
  {
    ExceptionStruct details; /*!<Structure of the exception.*/
  };

};

#endif
