// Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
//
// Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
// CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//
// See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
//

#ifndef  SALOMEDSCLIENT_HXX
#define  SALOMEDSCLIENT_HXX

#include "SALOMEDSClient_definitions.hxx"
#include "SALOMEDSClient_AttributeComment.hxx"
#include "SALOMEDSClient_AttributeDrawable.hxx"
#include "SALOMEDSClient_AttributeExpandable.hxx"
#include "SALOMEDSClient_AttributeExternalFileDef.hxx"
#include "SALOMEDSClient_AttributeFileType.hxx"
#include "SALOMEDSClient_AttributeFlags.hxx"
#include "SALOMEDSClient_AttributeGraphic.hxx"
#include "SALOMEDSClient_AttributeIOR.hxx"
#include "SALOMEDSClient_AttributeInteger.hxx"
#include "SALOMEDSClient_AttributeLocalID.hxx"
#include "SALOMEDSClient_AttributeName.hxx"
#include "SALOMEDSClient_AttributeOpened.hxx"
#include "SALOMEDSClient_AttributePersistentRef.hxx"
#include "SALOMEDSClient_AttributePixMap.hxx"
#include "SALOMEDSClient_AttributePythonObject.hxx"
#include "SALOMEDSClient_AttributeReal.hxx"
#include "SALOMEDSClient_AttributeSelectable.hxx"
#include "SALOMEDSClient_AttributeSequenceOfInteger.hxx"
#include "SALOMEDSClient_AttributeSequenceOfReal.hxx"
#include "SALOMEDSClient_AttributeStudyProperties.hxx"
#include "SALOMEDSClient_AttributeTableOfInteger.hxx"
#include "SALOMEDSClient_AttributeTableOfReal.hxx"
#include "SALOMEDSClient_AttributeTableOfString.hxx"
#include "SALOMEDSClient_AttributeTarget.hxx"
#include "SALOMEDSClient_AttributeTextColor.hxx"
#include "SALOMEDSClient_AttributeTextHighlightColor.hxx"
#include "SALOMEDSClient_AttributeTreeNode.hxx"
#include "SALOMEDSClient_AttributeUserID.hxx"
#include "SALOMEDSClient_AttributeParameter.hxx"
#include "SALOMEDSClient_AttributeString.hxx"
#include "SALOMEDSClient_ChildIterator.hxx"
#include "SALOMEDSClient_GenericAttribute.hxx"
#include "SALOMEDSClient_SComponent.hxx"
#include "SALOMEDSClient_SComponentIterator.hxx"
#include "SALOMEDSClient_SObject.hxx"
#include "SALOMEDSClient_Study.hxx"
#include "SALOMEDSClient_StudyBuilder.hxx"
#include "SALOMEDSClient_StudyManager.hxx"
#include "SALOMEDSClient_UseCaseBuilder.hxx"
#include "SALOMEDSClient_UseCaseIterator.hxx"

#endif
