// Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//
// See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
//

//  File   : SALOME_ParamPorts.idl
//  Author : Andre RIBES, EDF
//
#ifndef _SALOME_ParamPorts_IDL_
#define _SALOME_ParamPorts_IDL_

#include "SALOME_Ports.idl"

module Ports 
{
  /*!  \brief Interface of a port for parametric application
       This interface defines a Data_Port that sends a vector of
       double that are distributed along the different processor
       and returns a vector of double for the results
  */
  interface Param_Double_Port : Ports::Data_Port {
    typedef sequence<double>  seq_double;
    void put(in Ports::Param_Double_Port::seq_double param_data);
    void get_results(out Ports::Param_Double_Port::seq_double param_results);
  };
};
#endif
