// Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
//
// Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
// CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//
// See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
//

//  File   : Palm_Ports.idl
//  Author : André RIBES, EDF
//
#ifndef _PALM_PORTS_IDL_
#define _PALM_PORTS_IDL_

#include "SALOME_Ports.idl"

/*! \file  Palm_Ports.idl
    \brief This file contains IDL palm ports declarations.
*/

module Ports {

  module Palm_Ports {

    /*! \brief Interface of a short for palm.
     */
    interface Palm_Data_Short_Port : Ports::Data_Port {

      /*! \brief  This operation permits to transmit a short.

	\param data the short transmitted.
	\param time time associated to the short.
	\param tag  tag associated to the short.
       */
      void put (in short data, in long time, in long tag);
    };

    /*!
      It is a sequence of short for palm ports.
     */
    typedef sequence<short> seq_short;

    /*! \brief Interface of a sequence of short port.
     */
    interface Palm_Data_Seq_Short_Port : Ports::Data_Port {
      /*! \brief  This operation permits to transmit a sequence of short.

	\param data the sequence transmitted.
	\param time time associated to the data.
	\param tag  tag associated to the data.
       */
      void put (in seq_short data, in long time, in long tag);
    };

  };
};

#endif
