# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.40
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.
# This file is compatible with both classic and new-style classes.

"""
CALCIUM python wrapping : Superv_Component class

"""

from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_calcium', [dirname(__file__)])
        except ImportError:
            import _calcium
            return _calcium
        if fp is not None:
            try:
                _mod = imp.load_module('_calcium', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _calcium = swig_import_helper()
    del swig_import_helper
else:
    import _calcium
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static) or hasattr(self,name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


class intArray(_object):
    """Proxy of C++ intArray class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, intArray, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, intArray, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(self, size_t nelements) -> intArray"""
        this = _calcium.new_intArray(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _calcium.delete_intArray
    __del__ = lambda self : None;
    def __getitem__(self, *args):
        """__getitem__(self, size_t index) -> int"""
        return _calcium.intArray___getitem__(self, *args)

    def __setitem__(self, *args):
        """__setitem__(self, size_t index, int value)"""
        return _calcium.intArray___setitem__(self, *args)

    def cast(self):
        """cast(self) -> int"""
        return _calcium.intArray_cast(self)

    def frompointer(*args):
        """frompointer(int t) -> intArray"""
        return _calcium.intArray_frompointer(*args)

    if _newclass:frompointer = staticmethod(frompointer)
    __swig_getmethods__["frompointer"] = lambda x: frompointer
intArray_swigregister = _calcium.intArray_swigregister
intArray_swigregister(intArray)

def intArray_frompointer(*args):
  """intArray_frompointer(int t) -> intArray"""
  return _calcium.intArray_frompointer(*args)

class longArray(_object):
    """Proxy of C++ longArray class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, longArray, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, longArray, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(self, size_t nelements) -> longArray"""
        this = _calcium.new_longArray(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _calcium.delete_longArray
    __del__ = lambda self : None;
    def __getitem__(self, *args):
        """__getitem__(self, size_t index) -> long"""
        return _calcium.longArray___getitem__(self, *args)

    def __setitem__(self, *args):
        """__setitem__(self, size_t index, long value)"""
        return _calcium.longArray___setitem__(self, *args)

    def cast(self):
        """cast(self) -> long"""
        return _calcium.longArray_cast(self)

    def frompointer(*args):
        """frompointer(long t) -> longArray"""
        return _calcium.longArray_frompointer(*args)

    if _newclass:frompointer = staticmethod(frompointer)
    __swig_getmethods__["frompointer"] = lambda x: frompointer
longArray_swigregister = _calcium.longArray_swigregister
longArray_swigregister(longArray)

def longArray_frompointer(*args):
  """longArray_frompointer(long t) -> longArray"""
  return _calcium.longArray_frompointer(*args)

class floatArray(_object):
    """Proxy of C++ floatArray class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, floatArray, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, floatArray, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(self, size_t nelements) -> floatArray"""
        this = _calcium.new_floatArray(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _calcium.delete_floatArray
    __del__ = lambda self : None;
    def __getitem__(self, *args):
        """__getitem__(self, size_t index) -> float"""
        return _calcium.floatArray___getitem__(self, *args)

    def __setitem__(self, *args):
        """__setitem__(self, size_t index, float value)"""
        return _calcium.floatArray___setitem__(self, *args)

    def cast(self):
        """cast(self) -> float"""
        return _calcium.floatArray_cast(self)

    def frompointer(*args):
        """frompointer(float t) -> floatArray"""
        return _calcium.floatArray_frompointer(*args)

    if _newclass:frompointer = staticmethod(frompointer)
    __swig_getmethods__["frompointer"] = lambda x: frompointer
floatArray_swigregister = _calcium.floatArray_swigregister
floatArray_swigregister(floatArray)

def floatArray_frompointer(*args):
  """floatArray_frompointer(float t) -> floatArray"""
  return _calcium.floatArray_frompointer(*args)

class doubleArray(_object):
    """Proxy of C++ doubleArray class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, doubleArray, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, doubleArray, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(self, size_t nelements) -> doubleArray"""
        this = _calcium.new_doubleArray(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _calcium.delete_doubleArray
    __del__ = lambda self : None;
    def __getitem__(self, *args):
        """__getitem__(self, size_t index) -> double"""
        return _calcium.doubleArray___getitem__(self, *args)

    def __setitem__(self, *args):
        """__setitem__(self, size_t index, double value)"""
        return _calcium.doubleArray___setitem__(self, *args)

    def cast(self):
        """cast(self) -> double"""
        return _calcium.doubleArray_cast(self)

    def frompointer(*args):
        """frompointer(double t) -> doubleArray"""
        return _calcium.doubleArray_frompointer(*args)

    if _newclass:frompointer = staticmethod(frompointer)
    __swig_getmethods__["frompointer"] = lambda x: frompointer
doubleArray_swigregister = _calcium.doubleArray_swigregister
doubleArray_swigregister(doubleArray)

def doubleArray_frompointer(*args):
  """doubleArray_frompointer(double t) -> doubleArray"""
  return _calcium.doubleArray_frompointer(*args)

class stringArray(_object):
    """Proxy of C++ stringArray class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, stringArray, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, stringArray, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(self, int nelements, int size = 0) -> stringArray
        __init__(self, int nelements) -> stringArray
        """
        this = _calcium.new_stringArray(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _calcium.delete_stringArray
    __del__ = lambda self : None;
    def __getitem__(self, *args):
        """__getitem__(self, int index) -> char"""
        return _calcium.stringArray___getitem__(self, *args)

    def __setitem__(self, *args):
        """__setitem__(self, int index, char value)"""
        return _calcium.stringArray___setitem__(self, *args)

    __swig_setmethods__["data"] = _calcium.stringArray_data_set
    __swig_getmethods__["data"] = _calcium.stringArray_data_get
    if _newclass:data = _swig_property(_calcium.stringArray_data_get, _calcium.stringArray_data_set)
    __swig_setmethods__["nelem"] = _calcium.stringArray_nelem_set
    __swig_getmethods__["nelem"] = _calcium.stringArray_nelem_get
    if _newclass:nelem = _swig_property(_calcium.stringArray_nelem_get, _calcium.stringArray_nelem_set)
    __swig_setmethods__["size"] = _calcium.stringArray_size_set
    __swig_getmethods__["size"] = _calcium.stringArray_size_get
    if _newclass:size = _swig_property(_calcium.stringArray_size_get, _calcium.stringArray_size_set)
stringArray_swigregister = _calcium.stringArray_swigregister
stringArray_swigregister(stringArray)

class DSC(_object):
    """Proxy of C++ Engines::DSC class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, DSC, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, DSC, name)
    __repr__ = _swig_repr
    AddingConnection = _calcium.DSC_AddingConnection
    RemovingConnection = _calcium.DSC_RemovingConnection
    ApplicationError = _calcium.DSC_ApplicationError
    def __init__(self): 
        """__init__(self) -> DSC"""
        this = _calcium.new_DSC()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _calcium.delete_DSC
    __del__ = lambda self : None;
DSC_swigregister = _calcium.DSC_swigregister
DSC_swigregister(DSC)

class PySupervCompo(_object):
    """Proxy of C++ PySupervCompo class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, PySupervCompo, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, PySupervCompo, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """
        __init__(self, CORBA::ORB_ptr orb, PortableServer::POA_ptr poa, Engines::Container_ptr contai, 
            char instanceName, 
            char interfaceName) -> PySupervCompo
        """
        this = _calcium.new_PySupervCompo(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _calcium.delete_PySupervCompo
    __del__ = lambda self : None;
    def init_service(self, *args):
        """init_service(self, char service_name) -> CORBA::Boolean"""
        return _calcium.PySupervCompo_init_service(self, *args)

    def create_provides_data_port(self, *args):
        """create_provides_data_port(self, char port_fab_type) -> provides_port"""
        return _calcium.PySupervCompo_create_provides_data_port(self, *args)

    def create_uses_data_port(self, *args):
        """create_uses_data_port(self, char port_fab_type) -> uses_port"""
        return _calcium.PySupervCompo_create_uses_data_port(self, *args)

    def add_port(self, *args):
        """
        add_port(self, char port_fab_type, char port_type, char port_name)
        add_port(self, provides_port port, char provides_port_name)
        add_port(self, uses_port port, char uses_port_name)
        """
        return _calcium.PySupervCompo_add_port(self, *args)

    def get_provides_port(self, *args):
        """get_provides_port(self, char provides_port_name, CORBA::Boolean connection_error) -> Ports::Port_ptr"""
        return _calcium.PySupervCompo_get_provides_port(self, *args)

    def connect_uses_port(self, *args):
        """connect_uses_port(self, char uses_port_name, Ports::Port_ptr provides_port_ref)"""
        return _calcium.PySupervCompo_connect_uses_port(self, *args)

    def connect_provides_port(self, *args):
        """connect_provides_port(self, char provides_port_name)"""
        return _calcium.PySupervCompo_connect_provides_port(self, *args)

    def disconnect_provides_port(self, *args):
        """disconnect_provides_port(self, char provides_port_name, Message message)"""
        return _calcium.PySupervCompo_disconnect_provides_port(self, *args)

    def disconnect_uses_port(self, *args):
        """
        disconnect_uses_port(self, char uses_port_name, Ports::Port_ptr provides_port_ref, 
            Message message)
        """
        return _calcium.PySupervCompo_disconnect_uses_port(self, *args)

    def get_port_properties(self, *args):
        """get_port_properties(self, char port_name) -> Ports::PortProperties_ptr"""
        return _calcium.PySupervCompo_get_port_properties(self, *args)

    def getInputFileToService(self, *args):
        """getInputFileToService(self, char service_name, char Salome_file_name) -> Engines::Salome_file_ptr"""
        return _calcium.PySupervCompo_getInputFileToService(self, *args)

    def checkInputFilesToService(self, *args):
        """checkInputFilesToService(self, char service_name)"""
        return _calcium.PySupervCompo_checkInputFilesToService(self, *args)

    def setInputFileToService(self, *args):
        """setInputFileToService(self, char service_name, char Salome_file_name) -> Engines::Salome_file_ptr"""
        return _calcium.PySupervCompo_setInputFileToService(self, *args)

    def getOutputFileToService(self, *args):
        """getOutputFileToService(self, char service_name, char Salome_file_name) -> Engines::Salome_file_ptr"""
        return _calcium.PySupervCompo_getOutputFileToService(self, *args)

    def checkOutputFilesToService(self, *args):
        """checkOutputFilesToService(self, char service_name)"""
        return _calcium.PySupervCompo_checkOutputFilesToService(self, *args)

    def setOutputFileToService(self, *args):
        """setOutputFileToService(self, char service_name, char Salome_file_name) -> Engines::Salome_file_ptr"""
        return _calcium.PySupervCompo_setOutputFileToService(self, *args)

    def add_provides_port(self, *args):
        """add_provides_port(self, Ports::Port_ptr ref, char provides_port_name, Ports::PortProperties_ptr port_prop)"""
        return _calcium.PySupervCompo_add_provides_port(self, *args)

    def add_uses_port(self, *args):
        """add_uses_port(self, char repository_id, char uses_port_name, Ports::PortProperties_ptr port_prop)"""
        return _calcium.PySupervCompo_add_uses_port(self, *args)

    def get_uses_port(self, *args):
        """get_uses_port(self, char uses_port_name) -> Engines::DSC::uses_port"""
        return _calcium.PySupervCompo_get_uses_port(self, *args)

    def is_connected(self, *args):
        """is_connected(self, char port_name) -> CORBA::Boolean"""
        return _calcium.PySupervCompo_is_connected(self, *args)

    def setTimeOut():
        """setTimeOut()"""
        return _calcium.PySupervCompo_setTimeOut()

    if _newclass:setTimeOut = staticmethod(setTimeOut)
    __swig_getmethods__["setTimeOut"] = lambda x: setTimeOut
    def ptr(self):
        """ptr(self) -> long"""
        return _calcium.PySupervCompo_ptr(self)

PySupervCompo_swigregister = _calcium.PySupervCompo_swigregister
PySupervCompo_swigregister(PySupervCompo)

def PySupervCompo_setTimeOut():
  """PySupervCompo_setTimeOut()"""
  return _calcium.PySupervCompo_setTimeOut()


def create_calcium_port(*args):
  """
    create_calcium_port(Superv_Component_i compo, char name, char type, char mode, 
        char depend)
    """
  return _calcium.create_calcium_port(*args)
TRUE = _calcium.TRUE
FALSE = _calcium.FALSE
FORMAT_TEMPS = _calcium.FORMAT_TEMPS
EPSILON = _calcium.EPSILON
VERSION_LEN = _calcium.VERSION_LEN
CODE_LEN = _calcium.CODE_LEN
VARIABLE_LEN = _calcium.VARIABLE_LEN
INSTANCE_LEN = _calcium.INSTANCE_LEN
DRIVER_LEN = _calcium.DRIVER_LEN
ARCHI_LEN = _calcium.ARCHI_LEN
ENV_VAR_LEN = _calcium.ENV_VAR_LEN
ACCC_LEN = _calcium.ACCC_LEN
PVM_LEN = _calcium.PVM_LEN
USER_LEN = _calcium.USER_LEN
MACHINE_LEN = _calcium.MACHINE_LEN
COMMAND_LEN = _calcium.COMMAND_LEN
MAX_LEN = _calcium.MAX_LEN
FICHIER_LEN = _calcium.FICHIER_LEN
NB_LIGNE_ERREUR = _calcium.NB_LIGNE_ERREUR
NB_LIGNE_TRACE = _calcium.NB_LIGNE_TRACE
CP_NORMALE = _calcium.CP_NORMALE
CP_ANORMALE = _calcium.CP_ANORMALE
CP_CONT = _calcium.CP_CONT
CP_ARRET = _calcium.CP_ARRET
CP_INITIAL = _calcium.CP_INITIAL
CP_NON_CONNECTE = _calcium.CP_NON_CONNECTE
CP_ATTENTE = _calcium.CP_ATTENTE
CP_EXECUTION = _calcium.CP_EXECUTION
CP_DECONNECTE = _calcium.CP_DECONNECTE
CP_TERMINE = _calcium.CP_TERMINE
CP_ENTIER = _calcium.CP_ENTIER
CP_REEL = _calcium.CP_REEL
CP_DREEL = _calcium.CP_DREEL
CP_COMPLEXE = _calcium.CP_COMPLEXE
CP_LOGIQUE = _calcium.CP_LOGIQUE
CP_CHAINE = _calcium.CP_CHAINE
CP_TEMPS = _calcium.CP_TEMPS
CP_ITERATION = _calcium.CP_ITERATION
CP_SEQUENTIEL = _calcium.CP_SEQUENTIEL
CP_IN = _calcium.CP_IN
CP_OUT = _calcium.CP_OUT
CP_NORMAL = _calcium.CP_NORMAL
CP_ESPION = _calcium.CP_ESPION
CP_ILLIMITE = _calcium.CP_ILLIMITE
CP_AUTESP = _calcium.CP_AUTESP
CP_SANS = _calcium.CP_SANS
CP_SUCCINCT = _calcium.CP_SUCCINCT
CP_DETAILLE = _calcium.CP_DETAILLE
CP_PAUSE = _calcium.CP_PAUSE
CP_LINEAIRE = _calcium.CP_LINEAIRE
CP_ESCALIER = _calcium.CP_ESCALIER
TI = _calcium.TI
TF = _calcium.TF
CP_TOUTES = _calcium.CP_TOUTES
CP_AUCUNE = _calcium.CP_AUCUNE
CP_PAS = _calcium.CP_PAS
CP_ROUTE = _calcium.CP_ROUTE
CP_ERREURS = _calcium.CP_ERREURS
CP_ROUTE_NORMAL = _calcium.CP_ROUTE_NORMAL
CP_ROUTE_DIRECT = _calcium.CP_ROUTE_DIRECT
CP_AUTO = _calcium.CP_AUTO
CP_BLOCAGE = _calcium.CP_BLOCAGE
CP_MANUEL = _calcium.CP_MANUEL
CP_IMMEDIATE = _calcium.CP_IMMEDIATE
CPLVAV = _calcium.CPLVAV
CPLVAF = _calcium.CPLVAF
CPLVAS = _calcium.CPLVAS
CPLCAV = _calcium.CPLCAV
CPLFAV = _calcium.CPLFAV
ERREUR = _calcium.ERREUR
CONNEXION = _calcium.CONNEXION
EMISSION = _calcium.EMISSION
DECONNEXION = _calcium.DECONNEXION
LECTURE_VARIABLE = _calcium.LECTURE_VARIABLE
ECRITURE_VARIABLE = _calcium.ECRITURE_VARIABLE
FIN_DE_PAS = _calcium.FIN_DE_PAS
AUIN_FIN_DE_PAS = _calcium.AUIN_FIN_DE_PAS
EFFACEMENT = _calcium.EFFACEMENT
MODE_EXEC = _calcium.MODE_EXEC
RUN_N_PAS = _calcium.RUN_N_PAS
DEF_CODE = _calcium.DEF_CODE
DEF_INSTANCE = _calcium.DEF_INSTANCE
DEF_LIEN = _calcium.DEF_LIEN
DEM_VERSION = _calcium.DEM_VERSION
SET_OPTION = _calcium.SET_OPTION
DEM_OPTION = _calcium.DEM_OPTION
DEM_CODES = _calcium.DEM_CODES
DEM_INSTS_DE_CODE = _calcium.DEM_INSTS_DE_CODE
DEM_VARIABLES = _calcium.DEM_VARIABLES
DEM_VARS_DE_CODE = _calcium.DEM_VARS_DE_CODE
DEM_VARS_DE_INST = _calcium.DEM_VARS_DE_INST
DEM_CARS_DE_VARIABLE = _calcium.DEM_CARS_DE_VARIABLE
DEM_CARS_DE_LIEN = _calcium.DEM_CARS_DE_LIEN
ENV_VALEURS_VARIABLE = _calcium.ENV_VALEURS_VARIABLE
ENV_OPTION = _calcium.ENV_OPTION
ENV_CODES = _calcium.ENV_CODES
ENV_INSTS_DE_CODE = _calcium.ENV_INSTS_DE_CODE
ENV_VARIABLES = _calcium.ENV_VARIABLES
ENV_VARS_DE_CODE = _calcium.ENV_VARS_DE_CODE
ENV_VARS_DE_INST = _calcium.ENV_VARS_DE_INST
ENV_CARS_DE_VARIABLE = _calcium.ENV_CARS_DE_VARIABLE
ENV_CARS_DE_LIEN = _calcium.ENV_CARS_DE_LIEN
ENV_TOPOLOGY = _calcium.ENV_TOPOLOGY
R_ENV_TOPOLOGY = _calcium.R_ENV_TOPOLOGY
ENV_MACHINE = _calcium.ENV_MACHINE
R_ENV_MACHINE = _calcium.R_ENV_MACHINE
ENV_CODE = _calcium.ENV_CODE
R_ENV_CODE = _calcium.R_ENV_CODE
ENV_INSTANCE = _calcium.ENV_INSTANCE
R_ENV_INSTANCE = _calcium.R_ENV_INSTANCE
ENV_VAR = _calcium.ENV_VAR
R_ENV_VAR = _calcium.R_ENV_VAR
ENV_LIEN = _calcium.ENV_LIEN
R_ENV_LIEN = _calcium.R_ENV_LIEN
ENV_ATTRIBUTS = _calcium.ENV_ATTRIBUTS
R_ENV_ATTRIBUTS = _calcium.R_ENV_ATTRIBUTS
ENV_VDATA = _calcium.ENV_VDATA
R_ENV_VDATA = _calcium.R_ENV_VDATA
P_HOST_DELETE = _calcium.P_HOST_DELETE
P_TASK_EXIT = _calcium.P_TASK_EXIT
CPOK = _calcium.CPOK
CPERIU = _calcium.CPERIU
CPNMVR = _calcium.CPNMVR
CPIOVR = _calcium.CPIOVR
CPTP = _calcium.CPTP
CPTPVR = _calcium.CPTPVR
CPIT = _calcium.CPIT
CPITVR = _calcium.CPITVR
CPRENA = _calcium.CPRENA
CPDNTP = _calcium.CPDNTP
CPDNDI = _calcium.CPDNDI
CPNMCD = _calcium.CPNMCD
CPNMIN = _calcium.CPNMIN
CPATTENTE = _calcium.CPATTENTE
CPBLOC = _calcium.CPBLOC
CPNTNULL = _calcium.CPNTNULL
CPLGVR = _calcium.CPLGVR
CPSTOP = _calcium.CPSTOP
CPATAL = _calcium.CPATAL
CPNOCP = _calcium.CPNOCP
CPCTVR = _calcium.CPCTVR
CPPASNULL = _calcium.CPPASNULL
CPMACHINE = _calcium.CPMACHINE
CPGRNU = _calcium.CPGRNU
CPGRIN = _calcium.CPGRIN
CPFINFICH = _calcium.CPFINFICH
CPERRFICH = _calcium.CPERRFICH
CPNORERR = _calcium.CPNORERR
CPRUNERR = _calcium.CPRUNERR
CPOPT = _calcium.CPOPT
CPVALOPT = _calcium.CPVALOPT
CPECREFF = _calcium.CPECREFF
CPLIEN = _calcium.CPLIEN
CPINARRET = _calcium.CPINARRET
CPSTOPSEQ = _calcium.CPSTOPSEQ
CPDECL = _calcium.CPDECL
CPINEXEC = _calcium.CPINEXEC
CPPVM = _calcium.CPPVM
CPERRINST = _calcium.CPERRINST
CPMODE = _calcium.CPMODE
CPINSTDEC = _calcium.CPINSTDEC
DEBUT_COUPLAGE = _calcium.DEBUT_COUPLAGE
FIN_COUPLAGE = _calcium.FIN_COUPLAGE
EXEC = _calcium.EXEC
CPCD = _calcium.CPCD
CPALLOW = _calcium.CPALLOW
CPECRI = _calcium.CPECRI
DEB_LECT = _calcium.DEB_LECT
FIN_LECT = _calcium.FIN_LECT
CPFIN = _calcium.CPFIN
CPFINP = _calcium.CPFINP
CPAUFP = _calcium.CPAUFP
CPINFP = _calcium.CPINFP
CPEFF = _calcium.CPEFF
STOP = _calcium.STOP
BLOCAGE = _calcium.BLOCAGE
CPPAUSE = _calcium.CPPAUSE
CPNORMAL = _calcium.CPNORMAL
CPNPAS = _calcium.CPNPAS
CPADCD = _calcium.CPADCD
CPADINCD = _calcium.CPADINCD
CPADLN = _calcium.CPADLN
CPIVERS = _calcium.CPIVERS
CPICD = _calcium.CPICD
CPIINCD = _calcium.CPIINCD
CPIVR = _calcium.CPIVR
CPIVRCD = _calcium.CPIVRCD
CPIVRIN = _calcium.CPIVRIN
CPICAVR = _calcium.CPICAVR
CPIILIEN = _calcium.CPIILIEN
CPSETOPT = _calcium.CPSETOPT
CPGETOPT = _calcium.CPGETOPT
TASK_EXIT = _calcium.TASK_EXIT
HOST_DELETE = _calcium.HOST_DELETE
ENTIER = _calcium.ENTIER
REEL = _calcium.REEL
DREEL = _calcium.DREEL
COMPLEXE = _calcium.COMPLEXE
LOGIQUE = _calcium.LOGIQUE
CHAINE = _calcium.CHAINE

def cp_cd(*args):
  """cp_cd(Superv_Component_i component) -> int"""
  return _calcium.cp_cd(*args)

def cp_een(*args):
  """
    cp_een(Superv_Component_i component, int dep, float t, int n, 
        char nom, int nval, int eval) -> int
    """
  return _calcium.cp_een(*args)

def cp_edb(*args):
  """
    cp_edb(Superv_Component_i component, int dep, double t, int n, 
        char nom, int nval, double eval) -> int
    """
  return _calcium.cp_edb(*args)

def cp_ere(*args):
  """
    cp_ere(Superv_Component_i component, int dep, float t, int n, 
        char nom, int nval, float eval) -> int
    """
  return _calcium.cp_ere(*args)

def cp_ecp(*args):
  """
    cp_ecp(Superv_Component_i component, int dep, float t, int n, 
        char nom, int nval, float ecpval) -> int
    """
  return _calcium.cp_ecp(*args)

def cp_elo(*args):
  """
    cp_elo(Superv_Component_i component, int dep, float t, int n, 
        char nom, int nval, int eval) -> int
    """
  return _calcium.cp_elo(*args)

def cp_ech(*args):
  """
    cp_ech(Superv_Component_i component, int dep, float t, int n, 
        char nom, int nval, char eval) -> int
    """
  return _calcium.cp_ech(*args)

def cp_elg(*args):
  """
    cp_elg(Superv_Component_i component, int dep, float t, int n, 
        char nom, int nval, long eval) -> int
    """
  return _calcium.cp_elg(*args)

def cp_eln(*args):
  """
    cp_eln(Superv_Component_i component, int dep, float t, int n, 
        char nom, int nval, long eval) -> int
    """
  return _calcium.cp_eln(*args)

def cp_len(*args):
  """
    cp_len(Superv_Component_i component, int dep, float ti, float tf, 
        int niter, char nom, int nmax, int lval) -> int
    """
  return _calcium.cp_len(*args)

def cp_ldb(*args):
  """
    cp_ldb(Superv_Component_i component, int dep, double ti, double tf, 
        int niter, char nom, int nmax, double lval) -> int
    """
  return _calcium.cp_ldb(*args)

def cp_lre(*args):
  """
    cp_lre(Superv_Component_i component, int dep, float ti, float tf, 
        int niter, char nom, int nmax, float lval) -> int
    """
  return _calcium.cp_lre(*args)

def cp_lcp(*args):
  """
    cp_lcp(Superv_Component_i component, int dep, float ti, float tf, 
        int niter, char nom, int nmax, float lcpval) -> int
    """
  return _calcium.cp_lcp(*args)

def cp_llo(*args):
  """
    cp_llo(Superv_Component_i component, int dep, float ti, float tf, 
        int niter, char nom, int nmax, int lval) -> int
    """
  return _calcium.cp_llo(*args)

def cp_lch(*args):
  """
    cp_lch(Superv_Component_i component, int dep, float ti, float tf, 
        int niter, char nom, int nmax, char lval) -> int
    """
  return _calcium.cp_lch(*args)

def cp_llg(*args):
  """
    cp_llg(Superv_Component_i component, int dep, float ti, float tf, 
        int niter, char nom, int nmax, long lval) -> int
    """
  return _calcium.cp_llg(*args)

def cp_lln(*args):
  """
    cp_lln(Superv_Component_i component, int dep, float ti, float tf, 
        int niter, char nom, int nmax, long lval) -> int
    """
  return _calcium.cp_lln(*args)

def cp_fini(*args):
  """cp_fini(Superv_Component_i component, char nom, int n) -> int"""
  return _calcium.cp_fini(*args)

def cp_fint(*args):
  """cp_fint(Superv_Component_i component, char nom, float t) -> int"""
  return _calcium.cp_fint(*args)

def cp_effi(*args):
  """cp_effi(Superv_Component_i component, char nom, int n) -> int"""
  return _calcium.cp_effi(*args)

def cp_efft(*args):
  """cp_efft(Superv_Component_i component, char nom, float t) -> int"""
  return _calcium.cp_efft(*args)

def cp_fin(*args):
  """cp_fin(Superv_Component_i component, int cp_end) -> int"""
  return _calcium.cp_fin(*args)


