// Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
//
// Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
// CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//
// See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
//

//=============================================================================
// File      : SALOME_MPITestComponent.idl
// Created   : mer jui 04 12:08:17 CEST 2003
// Author    : Bernard SECHER, CEA
// Project   : SALOME
// $Header: /home/server/cvs/KERNEL/KERNEL_SRC/idl/SALOME_TestMPIComponent.idl,v 1.7.2.1.22.3.12.1 2012-04-12 14:04:58 vsr Exp $
//=============================================================================
//
#include "SALOME_Component.idl"
#include "SALOME_MPIObject.idl"

/*! \file SALOME_TestMPIComponent.idl \brief interface to test MPIObject and MPIContainer
*/

module Engines
{
  
  interface TestMPIComponent : EngineComponent, MPIObject
  {
    // version synchrone (process 0)
    void Coucou(in long L);
    // version asynchrone (autres process)
    oneway void SPCoucou(in long L);
  };

};
