# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.40
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.
# This file is compatible with both classic and new-style classes.

from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_libNOTIFICATION', [dirname(__file__)])
        except ImportError:
            import _libNOTIFICATION
            return _libNOTIFICATION
        if fp is not None:
            try:
                _mod = imp.load_module('_libNOTIFICATION', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _libNOTIFICATION = swig_import_helper()
    del swig_import_helper
else:
    import _libNOTIFICATION
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static) or hasattr(self,name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


NOTIF_WARNING = _libNOTIFICATION.NOTIF_WARNING
NOTIF_STEP = _libNOTIFICATION.NOTIF_STEP
NOTIF_TRACE = _libNOTIFICATION.NOTIF_TRACE
NOTIF_VERBOSE = _libNOTIFICATION.NOTIF_VERBOSE
NOTIFICATION_ChannelName = _libNOTIFICATION.NOTIFICATION_ChannelName

def NOTIFICATION_date():
  """NOTIFICATION_date() -> char"""
  return _libNOTIFICATION.NOTIFICATION_date()

def NOTIFICATION_channel():
  """NOTIFICATION_channel() -> CosNA_EventChannel_ptr"""
  return _libNOTIFICATION.NOTIFICATION_channel()
class NOTIFICATION_Supplier_Swig(_object):
    """Proxy of C++ NOTIFICATION_Supplier_Swig class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, NOTIFICATION_Supplier_Swig, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, NOTIFICATION_Supplier_Swig, name)
    __repr__ = _swig_repr
    def __init__(self, *args): 
        """__init__(self, char instanceName, bool notif) -> NOTIFICATION_Supplier_Swig"""
        this = _libNOTIFICATION.new_NOTIFICATION_Supplier_Swig(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _libNOTIFICATION.delete_NOTIFICATION_Supplier_Swig
    __del__ = lambda self : None;
    def Send(self, *args):
        """Send(self, char graph, char node, char type, char message)"""
        return _libNOTIFICATION.NOTIFICATION_Supplier_Swig_Send(self, *args)

NOTIFICATION_Supplier_Swig_swigregister = _libNOTIFICATION.NOTIFICATION_Supplier_Swig_swigregister
NOTIFICATION_Supplier_Swig_swigregister(NOTIFICATION_Supplier_Swig)

class NOTIFICATION_Consumer_Swig(_object):
    """Proxy of C++ NOTIFICATION_Consumer_Swig class"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, NOTIFICATION_Consumer_Swig, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, NOTIFICATION_Consumer_Swig, name)
    __repr__ = _swig_repr
    def __init__(self): 
        """__init__(self) -> NOTIFICATION_Consumer_Swig"""
        this = _libNOTIFICATION.new_NOTIFICATION_Consumer_Swig()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _libNOTIFICATION.delete_NOTIFICATION_Consumer_Swig
    __del__ = lambda self : None;
    def Receive(self, *args):
        """
        Receive(self, char graph, char node, char type, char message, char sender, 
            long counter, char date, long stamp) -> bool
        """
        return _libNOTIFICATION.NOTIFICATION_Consumer_Swig_Receive(self, *args)

NOTIFICATION_Consumer_Swig_swigregister = _libNOTIFICATION.NOTIFICATION_Consumer_Swig_swigregister
NOTIFICATION_Consumer_Swig_swigregister(NOTIFICATION_Consumer_Swig)



