// Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
//
// Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
// CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//
// See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
//
//  File   : AddComponent.idl
//  Author : Jean Rahuel

#ifndef _ADDCOMPONENT_IDL
#define _ADDCOMPONENT_IDL

#include "SALOME_Component.idl"

module SuperVisionTest {

  interface Adder ;

  interface AddComponent : Engines::EngineComponent {
    double Add( in double x , in double y , out double z ) ;
    double AddWithoutSleep( in double x , in double y , out double z ) ;
    void Setx( in double x ) ;
    void Sety( in double y ) ;
    double Addxy() ;
    double AddyTox( in double y ) ;
    long Sigma( in long n ) ;
    double LastResult() ;

    Adder Addition() ;
    boolean AdditionObjRef1( out Adder anAdder ) ;
    void AdditionObjRef2( out boolean FuncValue , out Adder anAdder ) ;
    boolean AdditionObjRefs( in AddComponent AddComponent1 ,
                             in AddComponent Adder2 ,
                             in AddComponent Adder3 ,
                             out AddComponent RetAddComponent1 ,
                             out AddComponent RetAdder2 ,
                             out AddComponent RetAdder3 ) ;
  };

  interface Adder : Engines::EngineComponent {
    double Add( in double x , in double y , out double z ) ;
    double AddWithoutSleep( in double x , in double y , out double z ) ;
    double AddAndCompare( in double x , in double y , in Adder anOtherAdder,
                          out double z ) ;
    void SetLastResult( in double z ) ;
    void LastResult( out double z ) ;
    Engines::EngineComponent LccAddComponent( in string aContainer ,
                                              in string aComponentName ) ;
  };

} ;

#endif
